@@ -80,6 +80,9 @@ public:
     
     void fetchMoreFrames();
 
+private Q_SLOTS:
+    void stateChanged(KDevelop::IDebugSession::DebuggerState state);
+
 private:
     virtual void handleEvent(IDebugSession::event_t event);
 
@@ -88,6 +91,7 @@ private:
 
     int m_currentThread;
     int m_currentFrame;
+    bool m_updateCurrentFrameOnNextFetch;
 
     QList<ThreadItem> m_threads;
     QHash<int, QList<FrameItem> > m_frames;
