@@ -26,12 +26,4 @@ namespace KDevelop
 IDocumentation::~IDocumentation()
 {}
 
-QWidget* IDocumentation::documentationWidget(QWidget* parent)
-{
-    QTextBrowser* widget=new QTextBrowser(parent);
-    widget->setReadOnly(true);
-    widget->setText(description());
-    return widget;
-}
-
 }
