@@ -40,7 +40,6 @@
 #include "abstractnavigationcontext.h"
 #include "abstractdeclarationnavigationcontext.h"
 #include "navigationaction.h"
-#include "useswidget.h"
 #include "../../../interfaces/icore.h"
 #include "../../../interfaces/idocumentcontroller.h"
 #include <qapplication.h>
@@ -85,7 +84,11 @@ void AbstractNavigationWidget::initBrowser(int height) {
   m_browser = new KTextBrowser;
   
   // since we can embed arbitrary HTML we have to make sure it stays readable by forcing a black-white palette
-  m_browser->setPalette( QPalette( Qt::black, Qt::white ) );
+  QPalette p;
+  p.setColor(QPalette::AlternateBase, Qt::white);
+  p.setColor(QPalette::Base, Qt::white);
+  p.setColor(QPalette::Text, Qt::black);
+  m_browser->setPalette( p );
 
   m_browser->setOpenLinks(false);
   m_browser->setOpenExternalLinks(false);
@@ -221,17 +224,6 @@ void AbstractNavigationWidget::anchorClicked(const QUrl& url) {
     setContext( nextContext );
 }
 
-void AbstractNavigationWidget::executeContextAction(QString action) {
-  DUChainReadLocker lock( DUChain::lock() );
-  //We may get deleted while the call to acceptLink, so make sure we don't crash in that case
-  QPointer<AbstractNavigationWidget> thisPtr(this);
-  NavigationContextPointer oldContext = m_context;
-  NavigationContextPointer nextContext = m_context->executeLink(action);
-  
-  if(thisPtr)
-    setContext( nextContext );
-}
-
 void AbstractNavigationWidget::next() {
   DUChainReadLocker lock( DUChain::lock() );
   Q_ASSERT( m_context );
