@@ -107,7 +107,8 @@ public:
     RunController* q;
 
     QHash<KJob*, KAction*> jobs;
-    KActionMenu* stopAction;
+    KAction* stopAction;
+    KActionMenu* stopJobsMenu;
     KAction* profileAction;
     KAction* runAction;
     KAction* dbgAction;
@@ -185,8 +186,8 @@ public:
                 ILaunchConfiguration* ilaunch = 0;
                 foreach (LaunchConfiguration *l, launchConfigurations) {
                     QStringList path = l->config().readEntry(ConfiguredFromProjectItemEntry, QStringList());
-                    if (path == itemPath) {
-                        //kDebug() << "allready generated ilaunch" << ilaunch;
+                    if (l->type() == type && path == itemPath) {
+                        kDebug() << "allready generated ilaunch" << path;
                         ilaunch = l;
                         break;
                     }
@@ -444,15 +445,23 @@ void RunController::setupActions()
     ac->addAction("run_profile", d->profileAction);
     connect(d->profileAction, SIGNAL(triggered(bool)), this, SLOT(slotProfile()));
 
-    action = d->stopAction = new KActionMenu( KIcon("process-stop"), i18n("Stop Jobs"), this);
-    action->setIconText(i18nc("Short text for 'Stop Jobs' used in the toolbar", "Stop"));
-    action->setShortcut(Qt::Key_Escape);
+    action = d->stopAction = new KAction( KIcon("process-stop"), i18n("Stop All Jobs"), this);
+    action->setIconText(i18nc("Short text for 'Stop All Jobs' used in the toolbar", "Stop All"));
+    // Ctrl+Escape would be nicer, but thats taken by the ksysguard desktop shortcut
+    action->setShortcut(QKeySequence("Ctrl+Shift+Escape"));
     action->setToolTip(i18nc("@info:tooltip", "Stop all currently running jobs"));
-    action->setWhatsThis(i18nc("@info:whatsthis", "<b>Stop Jobs</b><p>Requests that all running jobs are stopped.</p>"));
+    action->setWhatsThis(i18nc("@info:whatsthis", "<b>Stop All Jobs</b><p>Requests that all running jobs are stopped.</p>"));
     action->setEnabled(false);
-    ac->addAction("run_stop", action);
+    ac->addAction("run_stop_all", action);
     connect(action, SIGNAL(triggered(bool)), this, SLOT(stopAllProcesses()));
 
+    action = d->stopJobsMenu = new KActionMenu( KIcon("process-stop"), i18n("Stop"), this);
+    action->setIconText(i18nc("Short text for 'Stop' used in the toolbar", "Stop"));
+    action->setToolTip(i18nc("@info:tooltip", "Menu allowing to stop individual jobs"));
+    action->setWhatsThis(i18nc("@info:whatsthis", "<b>Stop</b><p>List of Jobs that can be stopped individually.</p>"));
+    action->setEnabled(false);
+    ac->addAction("run_stop_menu", action);
+
     d->currentTargetAction = new KSelectAction( i18n("Current Launch Configuration"), this);
     d->currentTargetAction->setToolTip(i18nc("@info:tooltip", "Current Launch Configuration"));
     d->currentTargetAction->setStatusTip(i18n("Current Launch Configuration"));
@@ -547,7 +556,7 @@ void KDevelop::RunController::registerJob(KJob * job)
         if (Core::self()->setupFlags() != Core::NoUi) {
             stopJobAction = new KAction(job->objectName().isEmpty() ? i18n("<%1> Unnamed job", job->staticMetaObject.className()) : job->objectName(), this);
             stopJobAction->setData(QVariant::fromValue(static_cast<void*>(job)));
-            d->stopAction->addAction(stopJobAction);
+            d->stopJobsMenu->addAction(stopJobAction);
             connect (stopJobAction, SIGNAL(triggered(bool)), SLOT(slotKillJob()));
 
             job->setUiDelegate( new KDialogJobUiDelegate() );
@@ -600,8 +609,10 @@ void KDevelop::RunController::checkState()
         emit runStateChanged(d->state);
     }
 
-    if (Core::self()->setupFlags() != Core::NoUi)
+    if (Core::self()->setupFlags() != Core::NoUi) {
         d->stopAction->setEnabled(running);
+        d->stopJobsMenu->setEnabled(running);
+    }
 }
 
 void KDevelop::RunController::stopAllProcesses()
@@ -639,7 +650,20 @@ void KDevelop::RunController::finished(KJob * job)
             break;
 
         default:
-            KMessageBox::error(qApp->activeWindow(), job->errorString(), i18n("Process Error"));
+        {
+            ///WARNING: do *not* use a nested event loop here, it might cause
+            ///         random crashes later on, see e.g.:
+            ///         https://bugs.kde.org/show_bug.cgi?id=309811
+            KDialog* dialog = new KDialog;
+            dialog->setAttribute(Qt::WA_DeleteOnClose);
+            dialog->setWindowTitle(i18n("Process Error"));
+            dialog->setButtons(KDialog::Close);
+            KMessageBox::createKMessageBox(dialog, QMessageBox::Warning,
+                                           job->errorString(), QStringList(),
+                                           QString(), 0, KMessageBox::NoExec
+            );
+            dialog->show();
+        }
     }
 }
 
@@ -810,6 +834,28 @@ void RunController::setDefaultLaunch(ILaunchConfiguration* l)
     }
 }
 
+bool launcherNameExists(const QString& name)
+{
+    foreach(ILaunchConfiguration* config, Core::self()->runControllerInternal()->launchConfigurations()) {
+        if(config->name()==name)
+            return true;
+    }
+    return false;
+}
+
+QString makeUnique(const QString& name)
+{
+    if(launcherNameExists(name)) {
+        for(int i=2; ; i++) {
+            QString proposed = QString("%1 (%2)").arg(name).arg(i);
+            if(!launcherNameExists(proposed)) {
+                return proposed;
+            }
+        }
+    }
+    return name;
+}
+
 ILaunchConfiguration* RunController::createLaunchConfiguration ( LaunchConfigurationType* type,
                                                                  const QPair<QString,QString>& launcher,
                                                                  IProject* project, const QString& name )
@@ -836,8 +882,10 @@ ILaunchConfiguration* RunController::createLaunchConfiguration ( LaunchConfigura
     QString cfgName = name;
     if( name.isEmpty() )
     {
-        cfgName = i18n("New %1 Configuration", type->name() );
+        cfgName = i18n("New %1 Launcher", type->name() );
+        cfgName = makeUnique(cfgName);
     }
+    
     launchConfigGroup.writeEntry(LaunchConfiguration::LaunchConfigurationNameEntry, cfgName );
     launchConfigGroup.writeEntry(LaunchConfiguration::LaunchConfigurationTypeEntry, type->id() );
     launchConfigGroup.sync();
@@ -882,7 +930,7 @@ ContextMenuExtension RunController::contextMenuExtension ( Context* ctx )
                             hasLauncher = true;
                         }
                     }
-                    if( type->canLaunch(itm) && hasLauncher )
+                    if( hasLauncher && type->canLaunch(itm) )
                     {
                         d->launchAsInfo[i] = qMakePair( type->id(), mode->id() );
                         KAction* act = new KAction( d->launchAsMapper );
