@@ -24,6 +24,7 @@
 #include <QtGui/QStylePainter>
 #include <QtGui/QStackedWidget>
 #include <QtGui/QStyleOptionTabBarBase>
+#include <QtGui/QToolButton>
 #include <QtGui/qstyle.h>
 
 #include <kdebug.h>
@@ -37,6 +38,7 @@
 #include <kacceleratormanager.h>
 #include <kmenu.h>
 #include <kicon.h>
+#include <kmimetype.h>
 
 #include "view.h"
 #include "document.h"
@@ -81,6 +83,11 @@ class ContainerTabBar : public KTabBar {
     Container* m_container;
 };
 
+bool sortViews(const View* const lhs, const View* const rhs)
+{
+        return lhs->document()->title().compare(rhs->document()->title(), Qt::CaseInsensitive) < 0;
+}
+
 struct ContainerPrivate {
     QMap<QWidget*, View*> viewForWidget;
 
@@ -90,6 +97,42 @@ struct ContainerPrivate {
     QLabel *fileStatus;
     KSqueezedTextLabel *statusCorner;
     QWeakPointer<QWidget> leftCornerWidget;
+    QToolButton* documentListButton;
+    QMenu* documentListMenu;
+    QMap<View*, QAction*> documentListActionForView;
+
+    /**
+     * Updates the context menu which is shown when
+     * the document list button in the tab bar is clicked.
+     *
+     * It shall build a popup menu which contains all currently
+     * enabled views using the title their document provides.
+     */
+    void updateDocumentListPopupMenu()
+    {
+        qDeleteAll(documentListActionForView);
+        documentListActionForView.clear();
+        documentListMenu->clear();
+
+        // create a lexicographically sorted list
+        QVector<View*> views;
+        views.reserve(viewForWidget.size());
+
+        foreach(View* view, viewForWidget){ 
+            views << view;
+        }
+
+        qSort(views.begin(), views.end(), sortViews);
+
+        foreach(View* view, views) {
+            QAction* action = documentListMenu->addAction(view->document()->title());
+            action->setData(QVariant::fromValue(view));
+            documentListActionForView[view] = action;
+            action->setIcon(view->document()->icon());
+            ///FIXME: push this code somehow into shell, such that we can access the project model for
+            ///       icons and also get a neat, short path like the document switcher.
+        }
+    }
 };
 
 class UnderlinedLabel: public KSqueezedTextLabel {
@@ -165,6 +208,15 @@ Container::Container(QWidget *parent)
     m_tabBarLayout->setMargin(0);
     m_tabBarLayout->setSpacing(0);
 
+    d->documentListMenu = new QMenu(this);
+    d->documentListButton = new QToolButton(this);
+    d->documentListButton->setIcon(KIcon("format-list-unordered"));
+    d->documentListButton->setMenu(d->documentListMenu);
+    d->documentListButton->setPopupMode(QToolButton::InstantPopup);
+    d->documentListButton->setAutoRaise(true);
+    d->documentListButton->setToolTip(i18n("Show sorted list of opened documents"));
+    d->documentListButton->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Preferred);
+    m_tabBarLayout->addWidget(d->documentListButton);
     d->tabBar = new ContainerTabBar(this);
     d->tabBar->setContextMenuPolicy(Qt::CustomContextMenu);
     m_tabBarLayout->addWidget(d->tabBar);
@@ -186,6 +238,7 @@ Container::Container(QWidget *parent)
     connect(d->tabBar, SIGNAL(wheelDelta(int)), this, SLOT(wheelScroll(int)));
     connect(d->tabBar, SIGNAL(contextMenu(int,QPoint)), this, SLOT(contextMenu(int,QPoint)));
     connect(d->tabBar, SIGNAL(mouseMiddleClick(int)), this, SLOT(closeRequest(int)));
+    connect(d->documentListMenu, SIGNAL(triggered(QAction*)), this, SLOT(documentListActionTriggered(QAction*)));
 
     KConfigGroup group = KGlobal::config()->group("UiSettings");
     setTabBarHidden(group.readEntry("TabBarVisibility", 1) == 0);
@@ -269,13 +322,19 @@ void Container::addWidget(View *view, int position)
     d->tabBar->insertTab(idx, view->document()->statusIcon(), view->document()->title());
     Q_ASSERT(view);
     d->viewForWidget[w] = view;
+
+    // Update document list context menu. This has to be called before
+    // setCurrentWidget, because we call the status icon and title update slots
+    // already, which in turn need the document list menu to be setup.
+    d->updateDocumentListPopupMenu();
+
     setCurrentWidget(d->stack->currentWidget());
-    
+
     // This fixes a strange layouting bug, that could be reproduced like this: Open a few files in KDevelop, activate the rightmost tab.
     // Then temporarily switch to another area, and then switch back. After that, the tab-bar was gone.
     // The problem could only be fixed by closing/opening another view.
     d->tabBar->setMinimumHeight(d->tabBar->sizeHint().height());
-    
+
     connect(view, SIGNAL(statusChanged(Sublime::View*)), this, SLOT(statusChanged(Sublime::View*)));
     connect(view->document(), SIGNAL(statusIconChanged(Sublime::Document*)), this, SLOT(statusIconChanged(Sublime::Document*)));
     connect(view->document(), SIGNAL(titleChanged(Sublime::Document*)), this, SLOT(documentTitleChanged(Sublime::Document*)));
@@ -297,6 +356,11 @@ void Container::statusIconChanged(Document* doc)
             if (tabIndex != -1) {
                 d->tabBar->setTabIcon(tabIndex, doc->statusIcon());
             }
+
+            // Update the document title's menu associated action
+            // using the View* index map
+            Q_ASSERT(d->documentListActionForView.contains(it.value()));
+            d->documentListActionForView[it.value()]->setIcon(doc->icon());
             break;
         }
     }
@@ -326,6 +390,10 @@ void Container::documentTitleChanged(Sublime::Document* doc)
             if (tabIndex != -1) {
                 d->tabBar->setTabText(tabIndex, doc->title());
             }
+
+            // Update document list popup title
+            Q_ASSERT(d->documentListActionForView.contains(it.value()));
+            d->documentListActionForView[it.value()]->setText(doc->title());
             break;
         }
     }
@@ -394,6 +462,10 @@ void Sublime::Container::removeWidget(QWidget *w)
             disconnect(view->document(), SIGNAL(titleChanged(Sublime::Document*)), this, SLOT(documentTitleChanged(Sublime::Document*)));
             disconnect(view->document(), SIGNAL(statusIconChanged(Sublime::Document*)), this, SLOT(statusIconChanged(Sublime::Document*)));
             disconnect(view, SIGNAL(statusChanged(Sublime::View*)), this, SLOT(statusChanged(Sublime::View*)));
+
+            // Update document list context menu
+            Q_ASSERT(d->documentListActionForView.contains(view));
+            delete d->documentListActionForView.take(view);
         }
     }
 }
@@ -489,6 +561,14 @@ QRect Container::tabRect(int tab) const
     return d->tabBar->tabRect(tab).translated(d->tabBar->mapToGlobal(QPoint(0, 0)));
 }
 
+void Container::documentListActionTriggered(QAction* action)
+{
+    Sublime::View* view = action->data().value< Sublime::View* >();
+    Q_ASSERT(view);
+    QWidget* widget = d->viewForWidget.key(view);
+    Q_ASSERT(widget);
+    setCurrentWidget(widget);
+}
 
 
 }
