@@ -246,9 +246,11 @@ void VcsPluginHelper::revert()
     foreach(const KUrl& url, d->ctxUrls) {
         IDocument* doc=ICore::self()->documentController()->documentForUrl(url);
         
-        if(doc) {
-            KTextEditor::ModificationInterface* modif=dynamic_cast<KTextEditor::ModificationInterface*>(doc->textDocument());
-            modif->setModifiedOnDiskWarning(false);
+        if(doc && doc->textDocument()) {
+            KTextEditor::ModificationInterface* modif = dynamic_cast<KTextEditor::ModificationInterface*>(doc->textDocument());
+            if (modif) {
+                modif->setModifiedOnDiskWarning(false);
+            }
             doc->textDocument()->setModified(false);
         }
     }
@@ -353,7 +355,8 @@ void VcsPluginHelper::history(const VcsRevision& rev)
 
     KDialog* dlg = new KDialog();
     dlg->setButtons(KDialog::Close);
-    dlg->setCaption(i18n("%2 History (%1)", url.pathOrUrl(), iface->name()));
+    dlg->setCaption(i18nc("%1: path or URL, %2: name of a version control system",
+                          "%2 History (%1)", url.pathOrUrl(), iface->name()));
     KDevelop::VcsEventWidget* logWidget = new KDevelop::VcsEventWidget(url, job, dlg);
     dlg->setMainWidget(logWidget);
     dlg->show();
@@ -431,21 +434,27 @@ void VcsPluginHelper::annotationContextMenuAboutToShow( KTextEditor::View* view,
     VcsAnnotationModel* model = qobject_cast<VcsAnnotationModel*>( annotateiface->annotationModel() );
     Q_ASSERT(model);
 
-    if(menu->actions().count()<3) {
-        VcsRevision rev = model->revisionForLine(line);
-        d->diffForRevAction->setData(QVariant::fromValue(rev));
-        d->diffForRevGlobalAction->setData(QVariant::fromValue(rev));
-        menu->addSeparator();
-        menu->addAction(d->diffForRevAction);
-        menu->addAction(d->diffForRevGlobalAction);
-        menu->addAction(new FlexibleAction(KIcon("edit-copy"), i18n("Copy Revision"), new CopyFunction(rev.revisionValue().toString()), menu));
-        menu->addAction(new FlexibleAction(KIcon("view-history"), i18n("Revision History..."), new HistoryFunction(this, rev), menu));
+    VcsRevision rev = model->revisionForLine(line);
+    // check if the user clicked on a row without revision information
+    if (rev.revisionType() == VcsRevision::Invalid) {
+        // in this case, do not action depending on revision informations
+        return;
     }
+
+    d->diffForRevAction->setData(QVariant::fromValue(rev));
+    d->diffForRevGlobalAction->setData(QVariant::fromValue(rev));
+    menu->addSeparator();
+    menu->addAction(d->diffForRevAction);
+    menu->addAction(d->diffForRevGlobalAction);
+    menu->addAction(new FlexibleAction(KIcon("edit-copy"), i18n("Copy Revision"), new CopyFunction(rev.revisionValue().toString()), menu));
+    menu->addAction(new FlexibleAction(KIcon("view-history"), i18n("History..."), new HistoryFunction(this, rev), menu));
 }
 
 void VcsPluginHelper::annotationVisibilityChange(KTextEditor::View* view, QMenu* menu, int visible)
 {
-    
+    Q_UNUSED(view);
+    Q_UNUSED(menu);
+    Q_UNUSED(visible);
 }
 
 void VcsPluginHelper::update()
@@ -466,7 +475,7 @@ void VcsPluginHelper::commit()
     KUrl url = d->ctxUrls.first();
     
     // We start the commit UI no matter whether there is real differences, as it can also be used to commit untracked files
-    VCSCommitDiffPatchSource* patchSource = new VCSCommitDiffPatchSource(new VCSStandardDiffUpdater(d->vcs, url), url, d->vcs);
+    VCSCommitDiffPatchSource* patchSource = new VCSCommitDiffPatchSource(new VCSStandardDiffUpdater(d->vcs, url));
     
     bool ret = showVcsDiff(patchSource);
 
